package com.fastbee.ggroup.service.impl;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fastbee.common.exception.ServiceException;
import com.fastbee.ggroup.domain.GSiteGroups;
import com.fastbee.ggroup.domain.dto.GGroupSiteDto;
import com.fastbee.ggroup.domain.dto.GGroupSiteRelateDto;
import com.fastbee.ggroup.domain.dto.GSitesEditDto;
import com.fastbee.ggroup.mapper.GSiteGroupsMapper;

import com.github.yulichang.toolkit.JoinWrappers;
import com.github.yulichang.wrapper.MPJLambdaWrapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fastbee.ggroup.mapper.GSitesMapper;
import com.fastbee.ggroup.domain.GSites;
import com.fastbee.ggroup.service.IGSitesService;
import org.springframework.transaction.annotation.Transactional;

/**
 * 站点Service业务层处理
 * 
 * @author kerwincui
 * @date 2024-09-29
 */
@Service
public class GSitesServiceImpl extends ServiceImpl<GSiteGroupsMapper,GSiteGroups> implements IGSitesService {
    @Autowired
    private GSitesMapper gSitesMapper;

    @Autowired
    private GSiteGroupsMapper gSiteGroupsMapper;

    /**
     * 查询站点
     *
     * @param id 站点主键
     * @return 站点
     */
    @Override
    public GSites selectGSitesById(Long id) {
        return gSitesMapper.selectGSitesById(id);
    }

    /**
     * 查询全部站点列表
     *
     * @param gSites 站点
     * @return 站点
     */
    @Override
    public List<GSites> selectGSitesList(GSites gSites) {

        return gSitesMapper.selectList(new LambdaQueryWrapper<GSites>()
                .select(GSites::getId, GSites::getName, GSites::getIcon, GSites::getType)
                .eq(GSites::getProjectId, 1)
        );
    }

    /**
     * 新增站点
     *
     * @param gGroupSiteDto 站点
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertGSites(GGroupSiteDto gGroupSiteDto) {

        GSites gSites = new GSites();
        gSites.setName(gGroupSiteDto.getName().trim());
        gSites.setType(gGroupSiteDto.getType());
        gSites.setIcon(gGroupSiteDto.getIcon());
        gSites.setProjectId(gGroupSiteDto.getProjectId());
        //根据名称判断是否存在该站点
        GSites gSites1 = gSitesMapper.selectOne(new LambdaQueryWrapper<GSites>()
                .select(GSites::getName, GSites::getId)
                .eq(GSites::getName, gGroupSiteDto.getName().trim()));
        if (gSites1 != null) {
            throw new ServiceException("站点名称已存在！");
        }
        //插入站点表
        int inserted = gSitesMapper.insert(gSites);
        gGroupSiteDto.setSiteId(gSites.getId());
        //插入关系表
        int inserted1 = gSitesMapper.insertGSites(gGroupSiteDto);
        return inserted == 1 && inserted1 == 1 ? 1 : 0;
    }

    /**
     * 修改站点
     *
     * @param gSitesEditDto 站点
     * @return 结果
     */
    @Override
    public int updateGSites(GSitesEditDto gSitesEditDto) {
        //查修改前站点名称
        GSites gSites = gSitesMapper.selectGSitesById(gSitesEditDto.getId());
        if (Objects.isNull(gSites)) {
            throw new ServiceException("修改站点不存在！");
        }
        //判断修改后是否和修改前一致
        if(gSites.getName().trim().equals(gSitesEditDto.getName().trim())){
            return 1;
        }
        //不能有重复名字的站点
        GSites gSites1 = gSitesMapper.selectOne(new LambdaQueryWrapper<GSites>()
                .select(GSites::getName, GSites::getId)
                .eq(GSites::getName, gSitesEditDto.getName().trim()));

        if (Objects.nonNull(gSites1)) {
            throw new ServiceException("站点名称已存在！");
        }
        GSites sites = new GSites();
        BeanUtils.copyProperties(gSitesEditDto, sites);
        return gSitesMapper.updateById(sites);
    }

    /**
     * 批量删除站点
     *
     * @param ids 需要删除的站点主键
     * @return 结果
     */
    @Override
    public int deleteGSitesByIds(Long[] ids) {
        return gSitesMapper.deleteGSitesByIds(ids);
    }

    /**
     * 删除站点信息
     *
     * @param id 站点主键
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteGSitesById(Long id) {
        GSiteGroups gSiteGroups = gSiteGroupsMapper.selectGSiteGroupsById(id);
        if (Objects.isNull(gSiteGroups)) {
            throw new ServiceException("站点不存在");
        }
//        int deleted = gSitesMapper.deleteById(gSiteGroups.getSiteId());
        //删除关系
        return gSiteGroupsMapper.deleteById(id);
    }


    @Override
    public int relateGroup(GGroupSiteRelateDto gGroupSiteRelateDto) {
        Long parentId = gGroupSiteRelateDto.getParentId();
        if (StringUtils.isBlank(parentId.toString())) {
            throw new ServiceException("请选择分组!");
        }
        List<Long> siteIds = gGroupSiteRelateDto.getSiteIds();
        if (siteIds.isEmpty()) {
            throw new ServiceException("请选择站点!");
        }
        super.saveBatch(siteIds
                .stream()
                .map(this::setSiteIdToSiteGroups)
                .collect(Collectors.toList())
                .stream()
                .map(item -> {
                    item.setParentId(parentId);
                    item.setProjectId(1L);
                    return item;
                })
                .collect(Collectors.toList()));
        return 1;
    }

    @Override
    public List<GSites> searchGSitesList(GSites gSites) {
        return gSitesMapper.selectList(new LambdaQueryWrapper<GSites>()
                        .eq(GSites::getProjectId, gSites.getProjectId())
                        .select(GSites::getId, GSites::getName, GSites::getType, GSites::getIcon)
                        .like(GSites::getName, gSites.getSearchValue()));
    }

    private GSiteGroups setSiteIdToSiteGroups(Long siteId) {
        GSiteGroups gSiteGroups = new GSiteGroups();
        gSiteGroups.setSiteId(siteId);
        return gSiteGroups;
    }

}

